#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@文件名称: django_deploy.py
@功能描述: Django项目部署自动化脚本
@参数说明: 无
@依赖环境: Python >= 3.6, Django, MySQL客户端
@使用方法: python django_deploy.py
"""
import os
import re
import sys
import subprocess
import shutil
from pathlib import Path

# 终端颜色代码（不依赖外部包）
class TermColors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

def color_print(text, color):
    """彩色打印输出"""
    print(f"{color}{text}{TermColors.ENDC}")

def get_project_root():
    """获取项目根目录（包含manage.py的目录）"""
    current = Path.cwd()
    for path in [current] + list(current.parents):
        if (path / "manage.py").exists():
            return path
    raise FileNotFoundError("未找到Django项目根目录（找不到manage.py）")

def get_project_name():
    """获取项目名称"""
    root = get_project_root()
    settings_dir = root / root.name
    if (settings_dir / "settings.py").exists():
        return root.name
    return root.name

def clean_migrations():
    """清理所有app的migrations目录（除__init__.py外）"""
    root = get_project_root()
    color_print("➤ 正在清理迁移文件...", TermColors.OKBLUE)
    
    for item in root.iterdir():
        if item.is_dir() and (item / "migrations").exists():
            migrations_dir = item / "migrations"
            for migration in migrations_dir.glob("*.py"):
                if migration.name != "__init__.py":
                    migration.unlink()
                    color_print(f"  已删除: {migration}", TermColors.WARNING)
    color_print("✔ 迁移文件清理完成", TermColors.OKGREEN)

def create_database():
    """创建项目数据库"""
    project_name = get_project_name()
    env_vars = get_env_from_bashrc()
    mysql_pass = env_vars.get('MYSQL_ROOT_PASS', 'Chan1996!')
    
    color_print(f"➤ 正在创建数据库: {project_name}", TermColors.OKBLUE)
    try:
        subprocess.run(
            ["mysql", "-uroot", f"-p{mysql_pass}", "-e", f"CREATE DATABASE IF NOT EXISTS {project_name} CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci"],
            check=True
        )
        color_print(f"✔ 数据库 {project_name} 创建成功", TermColors.OKGREEN)
    except subprocess.CalledProcessError as e:
        color_print(f"✖ 数据库创建失败: {e}", TermColors.FAIL)
        sys.exit(1)

def get_env_from_bashrc():
    """从.bashrc获取环境变量"""
    env_vars = {}
    bashrc = Path.home() / ".bashrc"
    if not bashrc.exists():
        return env_vars
    
    with bashrc.open() as f:
        content = f.read()
    
    for match in re.finditer(r'export\s+([A-Z_]+)=["\']([^"\']+)["\']', content):
        env_vars[match.group(1)] = match.group(2)
    return env_vars

def run_management_command(command, success_msg, error_msg):
    """运行Django管理命令"""
    root = get_project_root()
    os.chdir(root)
    try:
        color_print(f"➤ 正在执行: python manage.py {command}", TermColors.OKBLUE)
        subprocess.run([sys.executable, "manage.py", *command.split()], check=True)
        color_print(f"✔ {success_msg}", TermColors.OKGREEN)
    except subprocess.CalledProcessError as e:
        color_print(f"✖ {error_msg}: {e}", TermColors.FAIL)
        sys.exit(1)

def summary(project_name):
    """显示部署总结"""
    root = get_project_root()
    static_path = root.parent / f"{project_name}_staticfiles"
    
    color_print("\n" + "="*50, TermColors.HEADER)
    color_print("✔ 部署完成总结", TermColors.HEADER)
    color_print("="*50, TermColors.HEADER)
    color_print(f"▪ 项目名称: {project_name}", TermColors.BOLD)
    color_print(f"▪ 项目目录: {root}", TermColors.BOLD)
    color_print(f"▪ 静态文件目录: {static_path}", TermColors.BOLD)
    color_print(f"▪ 数据库名称: {project_name}", TermColors.BOLD)
    color_print("="*50 + "\n", TermColors.HEADER)

def main():
    try:
        project_name = get_project_name()
        color_print(f"🔍 检测到项目: {project_name}", TermColors.HEADER)
        
        # 1. 创建数据库
        create_database()
        
        # 2. 清理迁移文件
        clean_migrations()
        
        # 3. 执行静态文件收集
        run_management_command(
            "collectstatic --noinput",
            "静态文件收集完成",
            "静态文件收集失败"
        )
        
        # 4. 生成迁移文件
        run_management_command(
            "makemigrations",
            "迁移文件生成完成",
            "迁移文件生成失败"
        )
        
        # 5. 执行数据库迁移
        run_management_command(
            "migrate",
            "数据库迁移完成",
            "数据库迁移失败"
        )
        
        # 显示总结
        summary(project_name)
        
    except Exception as e:
        color_print(f"✖ 发生错误: {e}", TermColors.FAIL)
        sys.exit(1)

if __name__ == "__main__":
    main()